import 'whatwg-fetch'

const DEFAULT_HEADERS = {
  Accept: 'application/json'
}

// NOTE: this will not work for nested objects
const jsonToQueryString = json => (Object.keys(json)
  .map((key) => {
    if (typeof key === 'object') {
      return jsonToQueryString(key)
    }

    return `${encodeURIComponent(key)}=${encodeURIComponent(json[key])}`
  })
  .join('&')
  .replace(/%20/g, '+'))

export async function request (url, { method = 'GET', data = {}, headers = {}, prefix = '' } = {}) {
  const credentials = 'same-origin'// send cookies in same-origin requests
  const requestObject = { credentials, method }
  let queryString = ''

  switch (method) {
    case 'GET': {
      const queryStringParams = jsonToQueryString(data)

      if (queryStringParams) {
        const queryStringPrefix = url.includes('?') ? '&' : '?'
        queryString = queryStringPrefix + queryStringParams
      }
      break
    }
    case 'DELETE':
    case 'POST':
    case 'PUT':
    case 'PATCH':
      requestObject.body = JSON.stringify(data)
      break
    default:
      throw new Error(`Unsupported method in http request: ${method}`)
  }

  requestObject.headers = {
    ...DEFAULT_HEADERS,
    ...headers
  }

  const fullUrl = prefix + url + queryString
  const req = new Request(fullUrl, requestObject) // eslint-disable-line no-undef

  let response

  try {
    response = await fetch(req) // eslint-disable-line no-undef
  } catch (error) {
    console.error('Network error!')
    throw error
  }

  let responseData
  try {
    // In case an API returns an empty body, we cannot use response.json() directly
    // If an empty body is returned, we will convert it to an empty JSON object
    responseData = await response.text()
    responseData = responseData.length ? JSON.parse(responseData) : {}
  } catch (error) {
    console.error('Failed parsing response', response)
    throw error
  }

  return {
    request: req,
    response,
    data: responseData
  }
}

export async function GET (url, { ...options }) {
  const method = 'GET'

  return await request(url, { method, ...options })
}

export async function POST (url, { ...options }) {
  const method = 'POST'

  return await request(url, { method, ...options })
}

export async function PUT (url, { ...options }) {
  const method = 'PUT'

  return await request(url, { method, ...options })
}

export async function PATCH (url, { ...options }) {
  const method = 'PATCH'

  return await request(url, { method, ...options })
}

export async function DELETE (url, { ...options }) {
  const method = 'DELETE'

  return await request(url, { method, ...options })
}
